#ifndef FEDITHEAD_H
#define FEDITHEAD_H
/*
* MPIA - GEIRS project
*
* "$Header: https://svn.mpia.de/gulli/geirs/src/branches/rjm/FEditHead.h 709 2013-10-29 19:43:10Z mathar $"
*
*/

#include "config.h"
#include <string>
#include <vector>

/** The CCfits library contained in
* <a href="http://heasarc.gsfc.nasa.gov/docs/software/fitsio/ccfits/"> CCfits 2.4</a>
*/
#include <CCfits/CCfits>

using namespace std ;
using namespace CCfits ;

/** The class edits header keywords FITS headers.
* The object is created specifying the name of an existing FITS file
* and the name of a template file which contains instructions to add,
* delete or modify keywords. A method then allows to run through these
* instructions and apply them to a specified primary or extension header.
*
* The main intend of this program is to eliminate the limitations
* known for the fmodhead, fthedit and eso2fits programs. The program
* <code>fedithead</code> does work with hierarchical keywords, and
* and it even allows to replace hierarchical keywords by simple keywords
* and vice versa. Groups of keywords that are pareseable by a regex(7)
* specification may also be removed or replaced with a syntax known from the sed(1),
* calculated with the aid of the <code>boost::regex</code> package.
* Moving keywords between HDU's or re-calculation of keywords as with ESO's
* <a href="http://archive.eso.org/ftu/FtuUserManual.ps">FTU</a> is not
* supported.
* @since 2012-10-18
* @author Richard J. Mathar
*/
class FEditHead : public FITS {

public:

	FEditHead(char *fitsname, char *fitstpl) ;
	FEditHead(char *fitsname, const vector<string> fitstpl) ;
	FEditHead(const char *gstream, int glines, int bitpix, int naxis, long* naxes, char fitstpl[], const char *fname) ;
	FEditHead(const char *gstream, int glines, int bitpix, int naxis, long* naxes, const vector<string> fitstpl, const char *fname) ;
	FEditHead(const char *gstream, char *fname, int glines= -1) ;

	~FEditHead() ;

	void exec(const bool verbose) ;

	/** Name of the configuration file with header template keywords
	*/
	String cfgFil ;

        char * allLines(int maxLines) ;

	/** The concatenation of all input lines, trimed/expanded to a multiple of 80
	* characters per line, one final \0.
	*/
	char *buffer ;

#ifdef FEDITHEAD_KEEP_FLAG
	/** Flag which signals that the FITS file is kept once the
	* object is destroyed.
	*/
	bool keepDtor ;
#endif

protected:

	/** The assembly of all template keyword ASCII lines. These
	* are all lines with the exception of empty or comment lines.
	* They are kept in the order of the configuration file, which
	* may be important if the same keyword may be targeted at more
	* than one place (in case of which the templates are enacted from
	* the top to the bottom of the configuation file.)
	*/
	vector<string> tplLins ;

	vector<string> scanTpl(const char *fitstpl, const bool raw80=false) const ;
	vector<string> scanTpl(const string fitstpl, const bool raw80=false) const ;
	vector<string> scanTpl(const vector<string>fitstpl, const bool raw80=false) const ;

	static string trimws(const string & instring) ;

	static char * parse_template(const string & instring) ;

	static int templateAnal(char * card, char keyname[], char keyval[], char cmt[], char dtyp[]) ;

	static vector<string> splitColon(const string & instring) ;
	static vector<string> splitColon(const string & instring, const string & delim) ;

	void readStream(const char *gstream, int glines, const bool restr=false) ;

	static bool addHduCard(HDU & hdu, char *card, const bool verbose = false) ;
private:

} ; /* FEditHead */

#endif /* FEDITHEAD_H */
